@@ -446,7 +446,7 @@ impl Record {
     ///
     /// let expected = Info::try_from(vec![
     ///     Field::new(Key::SamplesWithDataCount, Some(Value::Integer(3))),
-    ///     Field::new(Key::AlleleFrequencies, Some(Value::FloatArray(vec![0.5]))),
+    ///     Field::new(Key::AlleleFrequencies, Some(Value::FloatArray(vec![Some(0.5)]))),
     /// ])?;
     ///
     /// assert_eq!(record.info(), &expected);
@@ -479,7 +479,7 @@ impl Record {
     ///
     /// let expected = Info::try_from(vec![
     ///     Field::new(Key::SamplesWithDataCount, Some(Value::Integer(3))),
-    ///     Field::new(Key::AlleleFrequencies, Some(Value::FloatArray(vec![0.5]))),
+    ///     Field::new(Key::AlleleFrequencies, Some(Value::FloatArray(vec![Some(0.5)]))),
     ///     dp,
     /// ])?;
     ///
@@ -501,7 +501,7 @@ impl Record {
     /// };
     ///
     /// let keys = "GT:GQ".parse()?;
-    /// let genotypes = vec![Genotype::from_str_format("0|0:13", &keys)?];
+    /// let genotypes = vec![Genotype::from_str_keys("0|0:13", &keys)?];
     ///
     /// let record = vcf::Record::builder()
     ///     .set_chromosome("sq0".parse()?)
@@ -672,9 +672,9 @@ impl Record {
     pub fn end(&self) -> Result<Position, EndError> {
         use info::field::{Key, Value};
 
-        let end = if let Some(field) = self.info().get(&Key::EndPosition) {
-            match field.value() {
-                Some(Value::Integer(n)) => *n,
+        let end = if let Some(value) = self.info().get(&Key::EndPosition).and_then(|f| f.value()) {
+            match value {
+                Value::Integer(n) => *n,
                 _ => return Err(EndError::InvalidInfoEndPositionFieldValue),
             }
         } else {
@@ -697,7 +697,7 @@ impl fmt::Display for Record {
             f,
             "{chrom}\t{pos}\t{id}\t{ref}\t{alt}",
             chrom = self.chromosome(),
-            pos = i32::from(self.position()),
+            pos = self.position(),
             id = self.ids(),
             r#ref = self.reference_bases(),
             alt = self.alternate_bases(),
@@ -739,6 +739,18 @@ mod tests {
 
     #[test]
     fn test_end() -> Result<(), Box<dyn std::error::Error>> {
+        let record = Record::builder()
+            .set_chromosome("sq0".parse()?)
+            .set_position(Position::try_from(1)?)
+            .set_reference_bases("A".parse()?)
+            .set_info(Info::try_from(vec![info::Field::new(
+                info::field::Key::EndPosition,
+                None,
+            )])?)
+            .build()?;
+
+        assert_eq!(record.end(), Ok(Position::try_from(1)?));
+
         let record = Record::builder()
             .set_chromosome("sq0".parse()?)
             .set_position(Position::try_from(1)?)
@@ -783,7 +795,7 @@ mod tests {
         use genotypes::Genotype;
 
         let keys: genotypes::Keys = "GT:GQ".parse()?;
-        let genotypes = vec![Genotype::from_str_format("0|0:13", &keys)?];
+        let genotypes = vec![Genotype::from_str_keys("0|0:13", &keys)?];
 
         let record = Record::builder()
             .set_chromosome("sq0".parse()?)
